                  /*--.          .-"-. 
                 /   ._.        /  ` \
                 \_  (__\       \_v_/
                 //   \\        //   \\
                ((     ))      ((     ))
 --""---""----""---""--
                  |||            |||                                               
   Authors note:   |              |                                                
                                                                                   
   This file is part of work done by Emil Persson, aka Humus.                      
   You're free to use the code in any way you like, modified, unmodified           
   or cut'n'pasted into your own work. But for the good of the whole               
   programming community I will ask you to as long as possible and to the          
   extent you find suitable to follow these points:                                
    * If you use this file and it's contents unmodified, please leave this note.   
    * If you use a major part of the file in your app, please credit the author(s) 
      in one way or the other as you find suitable, for instance a small notice    
      in the readme file may do, or perhaps a link to the authors site.            
    * Share your work and ideas too as much as you can.                            
     _______                                                                       
   /` _____ `\;,    Humus                                                          
  (__(^===^)__)';,  emiper-8@student.luth.se                         ___           
    /  :::  \   ,;  ICQ #47010716                                  /^   ^\         
   |   :::   | ,;'                                                (     )        
 '._______.'`--oOo--(_)--oOo--*/

#ifndef _POLY_H_
#define _POLY_H_

class Poly {
protected:
	Vertex v[4];
	TextureID textureID, bumpMapID;
	Vertex s,t,normal;
	float scale;
	bool blend;
public:
	Poly(){}

	Poly(Vertex v0, Vertex v1, Vertex v2, TextureID texture, TextureID bumpMap, float scaleFactor){
		v[0] = v0;
		v[1] = v1;
		v[2] = v1 + v2 - v0;
		v[3] = v2;

		textureID = texture;
		bumpMapID = bumpMap;

		scale = scaleFactor;

		s = v1 - v0;
		t = v2 - v0;
		s.normalize();
		t.normalize();
		normal = cross(s,t);

		blend = false;		
	}

	void drawLit(TextureHandler *texHandler, SpotLight &light){
		int i;

		/* Quick and dirty way to speed things up a little.
		   Only checks against the plane, more efficient variants which checks
		   agaisnt the cone can be done, but I didn't bother for this simple demo.
		*/
		bool lit = false;
		for (i = 0; i < 4; i++){
			lit |= light.isPointInFrontOf(v[i]);
		}

		if (!lit) return;


		texHandler->setTextures(0, bumpMapID, 1, 2);
		texHandler->setTexEnvs(TEXENV_DOT3_NEXT, TEXENV_MODULATE_PRIMARY, TEXENV_MODULATE, TEXENV_MODULATE);
		if (blend) texHandler->setBlending(GL_ONE_MINUS_DST_COLOR, GL_ONE);
		texHandler->apply();

		glColor4fv((GLfloat *) &light.getColor());

		glBegin(GL_QUADS);
		for (i = 0; i < 4; i++){
			Vertex p = (light.getPosition() - v[i]) * 0.5f / light.getSize();
			glMultiTexCoord3f(GL_TEXTURE0_ARB, s * p + 0.5f, t * p + 0.5f, normal * p + 0.5f);
			glMultiTexCoord2f(GL_TEXTURE1_ARB, scale * (i == 1 || i == 2), scale * (i >= 2));

			glVertex3fv((GLfloat *) (v + i));
		}
		glEnd();

		blend = true;
	}

	void drawBase(TextureHandler *texHandler){
		// Modulate with the base texture. Add 0.13f for some ambient.
		texHandler->setTextures(textureID);
		if (blend) texHandler->setBlending(GL_SRC_ALPHA, GL_SRC_COLOR);
		texHandler->apply();

		if (blend){
			glColor4f(1,1,1,0.13f);
		} else {
			glColor3f(0.13f,0.13f,0.13f);
		}

		glBegin(GL_QUADS);
			glTexCoord2f(0,0);
			glVertex3fv((GLfloat *) (v + 0));
			glTexCoord2f(scale,0);
			glVertex3fv((GLfloat *) (v + 1));
			glTexCoord2f(scale,scale);
			glVertex3fv((GLfloat *) (v + 2));
			glTexCoord2f(0,scale);
			glVertex3fv((GLfloat *) (v + 3));
		glEnd();

		blend = false;
	}

};



#endif // _POLY_H_
